#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging

from octopus.core import op_codes

_MAX_RETRY_COUNT = 3

# Config option names
_FIRST_RECORD_OFFSET_OPTION = 'first_record_offset'
_NAME_OPTION = 'name'


class Crawler:

    def __init__(self, config, handler):
        # self.logger = logging.getLogger(self.__class__.__name__.lower())
        self.logger = logging.getLogger('core')
        self.source_name = config[_NAME_OPTION]
        self.first_entry_offset = int(config.get(_FIRST_RECORD_OFFSET_OPTION, 0))
        self.handler = handler
        self.retry_record_list = []
        self.retry_count = 0

    def crawl(self):
        allow_retry = self.retry_count <= _MAX_RETRY_COUNT
        counter = op_codes.OpCounter()
        # TODO : Il faudra envisager le cas des sources qui ne permettent pas de déterminer le nombre d'enregistrements
        # à l'avance.
        self._echo_start()
        record_count = self._count_records()
        print('-- START ({}) -> {} records --'.format(self.source_name, record_count), end='', flush=True)
        plotter = op_codes.Plotter()
        for record_num in range(1 + self.first_entry_offset, record_count + self.first_entry_offset + 1):
            plotter.print_line_head(record_num - 1 - self.first_entry_offset)
            record = self._read_record(record_num)
            op_code = self._handle_record(record, allow_retry)
            counter.inc(op_code)
            plotter.plot(op_code)
            if op_code == op_codes.RETRY:
                self.retry_record_list.append(record)
        print('\n-- END --')
        self._report(record_count, counter)
        if len(self.retry_record_list) > 0:
            self._crawl_retry_record_list()

    def _crawl_retry_record_list(self):
        self.retry_count += 1
        allow_retry = self.retry_count <= _MAX_RETRY_COUNT
        counter = op_codes.OpCounter()
        # TODO : Remplacer par un echo_start_retry
        # self._echo_start()
        record_list = self.retry_record_list
        self.retry_record_list = []
        record_count = len(record_list)
        print('-- RETRY ({}) -> {} records --'.format(self.source_name, record_count), end='', flush=True)
        plotter = op_codes.Plotter()
        for record_num in range(0, record_count):
            plotter.print_line_head(record_num)
            record = record_list[record_num]
            op_code = self._handle_record(record, allow_retry)
            counter.inc(op_code)
            plotter.plot(op_code)
            if op_code == op_codes.RETRY:
                self.retry_record_list.append(record)
        print('\n-- END --')
        # TODO: affiner le report pour les retry
        self._report(record_count, counter)
        if len(self.retry_record_list) > 0 and self.retry_count <= _MAX_RETRY_COUNT:
            self._crawl_retry_record_list()

    def _read_record(self, record_num):
        raise NotImplementedError()

    def _handle_record(self, record, allow_retry):
        return self.handler.handle(record, allow_retry=allow_retry)

    def _count_records(self):
        raise NotImplementedError()

    def _report(self, record_count, counter: op_codes.OpCounter):
        miss_count = record_count - counter.total()
        self.logger.info(
            'Crawling report: \n{}\n\tMISSED    : {}\n\n'.format(counter, str(miss_count).rjust(4)))

    def _echo_start(self):
        raise NotImplementedError()
