#!/usr/bin/python
# -*- coding: utf-8 -*-

import csv

from octopus.core.crawler import Crawler

_DEFAULT_ENCODING = 'ansi'
_DEFAULT_DELIMITER = ','
_DEFAULT_QUOTE_CHAR = '"'
_DEFAULT_COLUMN_MAP_DELIMITER = ','

_SOURCE_DELIMITER_KEY = 'source_delimiter'
_SOURCE_FILE_ENCODING_KEY = 'source_encoding'
_SOURCE_FILE_KEY = 'source_file'
_SOURCE_QUOTE_CHAR_KEY = 'source_quote_char'
_COLUMN_MAP_KEY = 'column_map'
_COLUMN_MAP_DELIMITER_KEY = 'column_map_delimiter'


def build(config, handler):
    return CSVCrawler(config, handler)


class CSVCrawler(Crawler):

    def __init__(self, config, handler):
        super().__init__(config, handler)
        encoding = config.get(_SOURCE_FILE_ENCODING_KEY, _DEFAULT_ENCODING)
        delimiter = config.get(_SOURCE_DELIMITER_KEY, _DEFAULT_DELIMITER)
        quote_char = config.get(_SOURCE_QUOTE_CHAR_KEY, _DEFAULT_QUOTE_CHAR)
        self.csv_file = config[_SOURCE_FILE_KEY]
        with open(self.csv_file, encoding=encoding) as source:
            self.rows = list(csv.reader(source, delimiter=delimiter, quotechar=quote_char))
            self.row_count = len(self.rows)
            self.column_count = len(self.rows[0])
        column_map_delimiter = config.get(_COLUMN_MAP_DELIMITER_KEY, _DEFAULT_COLUMN_MAP_DELIMITER)
        self.column_map = config[_COLUMN_MAP_KEY].split(column_map_delimiter)

    def _read_record(self, record_num):
        record = {}
        for col_num in range(0, self.column_count):
            record[self.column_map[col_num]] = self.rows[record_num][col_num]
        return record

    def _count_records(self):
        return self.row_count - 1 - self.first_entry_offset

    def _echo_start(self):
        self.logger.info(
            'Crawling {} from {} CSV file. \n Found {} rows by {} columns'.format(
                self.source_name, self.csv_file, self.row_count - self.first_entry_offset, self.column_count))
